#ifndef HMC_EDGE_H
#define HMC_EDGE_H

#include <maya/MPoint.h>
#include <maya/MMatrix.h>
#include <maya/MVector.h>

struct Face;
struct Vertex;

struct Edge {
   // Basics
   Edge(Vertex *_a, Vertex *_b);
   Edge(const Edge &rhs);
   const Edge &operator=(const Edge &rhs);

   // ModelPart
   void ComputeCost();

   // Instance Functions
   bool Contains(Vertex *v);
   int Collapse();
   void AddFace(Face *f);
   bool Valid();
   void Verify();
   void RemapTo(Edge *eNew, Face *fOld);
   void RemapVertex(Vertex *vOld, Vertex *vNew);
   void RemapFace(Face *fOld, Face *fNew);
   Face *GetOtherFace(const Face *const f) const;
   const bool ContainsFace(Face *f);
   Vertex *GetOtherVertex(Vertex *) const;

   // Helper functions
   bool findBestFit();
   bool findLineFit();
   bool findLocalFit();

   // Member Data
   MPoint newpos;
   MMatrix Q;
   MVector normal;
   mutable bool valid;
   double cost;
   bool removed;
   
   union {
      struct {
         Vertex *v0, *v1;
      };
      struct {
         Vertex *verts[2];
      };
   };

   union {
      struct {
         Face *f0, *f1;
      };
      struct {
         Face *faces[2];
      };
   };

};

struct edge_ptr_cmp {
   bool operator()(const Edge *a, const Edge *b) {
      return a->cost > b->cost;
   }
};

#endif