#include <iostream>
#include "Face.h"
#include "Edge.h"
#include "Vertex.h"

#include <maya/MPoint.h>
#include <maya/MVector.h>
#include <maya/MMatrix.h>

#include <assert.h>

using namespace std;

Face::Face(Vertex *_a, Vertex *_b, Vertex *_c) :
 a(_a),
 b(_b),
 c(_c),
 valid(false),
 removed(false) {
   a->faces.push_back(this);
   b->faces.push_back(this);
   c->faces.push_back(this);
}

void Face::SetEdge(int ndx, Edge *e) {
   assert (ndx >= 0 && ndx < 3);
   edges[ndx] = e;
   e->AddFace(this);
}

bool Face::ContainsEdge(Edge *e) {
   return x == e || y == e || z == e;
}

void Face::Invalidate() {
   valid = false;
}

void Face::ComputeCost() {
   MMatrix K;
   for (int i = 0; i < 4; i++)
      K(i,i) = 0.0;

   double ka, kb, kc, kd;
   ka = normal.x;
   kb = normal.y;
   kc = normal.z;
   
   kd = (-normal) * a->pos;

   K(0,0) = ka*ka;   K(0,1) = ka*kb;   K(0,2) = ka*kc;  K(0,3) = ka*kd;
   K(1,0) =K(0,1); K(1,1) = kb*kb;   K(1,2) = kb*kc;  K(1,3) = kb*kd;
   K(2,0) =K(0,2); K(2,1) =K(1,2); K(2,2) = kc*kc;  K(2,3) = kc*kd;
   K(3,0) =K(0,3); K(3,1) =K(1,3); K(3,2) =K(2,3);K(3,3) = kd*kd;

   for (int i = 0; i < 3; i++) {
      verts[i]->Q += K;
   }
}

void Face::RemapVertex(Vertex *vOld, Vertex *vNew) {
   if (a == vOld)
      a = vNew;
   else if (b == vOld)
      b = vNew;
   else if (c == vOld)
      c = vNew;
   else
      assert(0);
   vNew->faces.push_back(this);
}

void Face::RemapEdge(Edge *eOld, Edge *eNew) {
   if (x == eOld)
      x = eNew;
   if (y == eOld)
      y = eNew;
   if (z == eOld)
      z = eNew;
   else
      assert(0);
   valid = false;
}

void Face::Remove(Vertex *v) {
   if (removed) {
      cerr << "Remove called on a removed face" << endl;
      return;
   }

   if (x->removed) {
      // Merge 1 and 2
      if (y->Contains(v))
         z->RemapTo(y, this);
      else
         y->RemapTo(z, this);
   } else if (y->removed) {
      // Merge 0 and 2
      if (x->Contains(v))
         z->RemapTo(x, this);
      else
         x->RemapTo(z, this);
   } else if (z->removed) {
      if (x->Contains(v))
         y->RemapTo(x, this);
      else
         x->RemapTo(y, this);
   } else {
      cerr << "ERROR: Face::Remove called on a valid face" << endl;
   }
}

void Face::Verify() {
   if (removed)
      return;

   // TODO: Fill in Verify
}