#include <iostream>
#include "Vertex.h"
#include "Face.h"
#include "Edge.h"

#include <maya/MMatrix.h>
#include <maya/MPoint.h>
#include <maya/MVector.h>

using namespace std;

Vertex::Vertex() : 
   pos(MPoint()), 
   normal(MVector()),
   Q(MMatrix()),
   removed(false),
   faces(vector<Face *>()),
   edges(vector<Edge *>()) {
   for (int i = 0; i < 4; i++)
      Q(i,i) = 0.0;
}

Vertex::Vertex(const MPoint &_pos) : 
   pos(_pos), 
   normal(MVector()),
   Q(MMatrix()),
   removed(false),
   faces(vector<Face *>()),
   edges(vector<Edge *>()) {
   for (int i = 0; i < 4; i++)
      Q(i,i) = 0.0;
}

Vertex::Vertex(const Vertex &rhs) :
   pos(rhs.pos),
   normal(rhs.normal),
   Q(rhs.Q),
   removed(rhs.removed),
   faces(rhs.faces), //These probably won't work
   edges(rhs.edges) {

};

const Vertex& Vertex::operator=(const Vertex &rhs) {
   pos = rhs.pos;
   normal = rhs.normal;
   Q = rhs.Q;
   faces = rhs.faces;
   edges = rhs.edges;
   removed = rhs.removed;
   return *this;
}

void Vertex::ComputeCost() {
   MVector v = pos;
   Q = MMatrix::identity;

   Q(0,3) = -v.x;
   Q(1,3) = -v.y;
   Q(2,3) = -v.z;
   Q(3,3) = v.x*v.x + v.y*v.y + v.z*v.z;

   Q(3,0) = Q(0,3);
   Q(3,1) = Q(1,3);
   Q(3,2) = Q(2,3);
}

void Vertex::RemapEdge(Edge *eOld, Edge *eNew) {
   vector<Edge *>::iterator itr;
   for (edges.begin(); itr != edges.end(); itr++) {
      if (*itr == eOld) {
         *itr = eNew;
         break;
      }
   }
}

void Vertex::RemoveEdge(Edge *e) {
   vector<Edge *>::iterator itr;
   for (itr = edges.begin(); itr != edges.end(); itr++) {
      if (*itr == e) {
         edges.erase(itr);
         break;
      }
   }
}

void Vertex::RemoveFace(Face *f) {
   vector<Face *>::iterator itr;
   for (itr = faces.begin(); itr != faces.end(); itr++) {
      if (*itr == f) {
         faces.erase(itr);
         break;
      }
   }
}

void Vertex::RemapTo(Vertex *vNew) {
   if (vNew->removed) {
      cerr << "Remapping Vertex to removed vertex?" << endl;
      return;
   } else if (vNew == this) {
      cerr << "Remapping Vertex to self?" << endl;
      return;
   }

   removed = true;
   vNew->Q += Q;
   vNew->normal += normal;

   vector<Edge *>::iterator eItr;
   for (eItr = edges.begin(); eItr != edges.end(); eItr++) {
      if (!(*eItr)->removed)
         (*eItr)->RemapVertex(this, vNew);
   }

   vector<Face *>::iterator fItr;
   for (fItr = faces.begin(); fItr != faces.end(); fItr++) {
      if (!(*fItr)->removed)
         (*fItr)->RemapVertex(this,vNew);
   }

   for (eItr = vNew->edges.begin(); eItr != vNew->edges.end(); eItr++) {
      (*eItr)->ComputeCost();
   }
   edges.clear();
   faces.clear();
}